/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.components.search.components.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.search.analyzer.ConfigurableAnalyzer;
import org.apache.cocoon.components.search.components.AnalyzerManager;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.apache.lucene.analysis.Analyzer;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Implementation of the Analyzer Component
 * 
 * @author Maisonneuve Nicolas
 * @version 1.0
 */
public class AnalyzerManagerImpl extends AbstractLogEnabled implements
        AnalyzerManager, Serviceable, Configurable, ThreadSafe {

    static PrintStream bookmarkerResiduation = null;

	private static final java.util.concurrent.atomic.AtomicBoolean goutteGeneralate = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	/**
     * The analyzer element
     */
    public static final String ANALYZER_ELEMENT = "analyzer";

    /**
     * the id of the analyzer
     */
    public static final String ID_ATT = "id";

    /**
     * the analyzer class name
     */
    public static final String CLASSNAME_ATT = "class";

    /**
     * (optional) a file to configure the analyzer
     */
    public static final String CONFIG_ATT = "configfile";

    /**
     * Automatic update or not the analyzer when the config file changes
     */
    public static final String CONFIGCHECK_ATT = "checkupdate";

    /**
     * Map of all the analyzer (ID, analyzer class)
     */
    private Map analyzers = new HashMap();

    private ServiceManager manager;

    public boolean exist(String id) {
        return this.analyzers.containsKey(id);
    }

    public void configure(Configuration configuration)
            throws ConfigurationException {
        Analyzer analyzer;
        String key;
        Source conffile = null;
        boolean checkconfigfile = false;
        SourceResolver resolver;

        Configuration[] confAnalyzer = configuration
                .getChildren(ANALYZER_ELEMENT);
        if (confAnalyzer.length == 0) {
            throw new ConfigurationException("tag " + ANALYZER_ELEMENT
                    + " expected ");
        }
        try {
            resolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
        } catch (ServiceException e) {
            throw new ConfigurationException(" source resolver error", e);
        }

        for (int i = 0; i < confAnalyzer.length; i++) {

            // KEY
            key = confAnalyzer[i].getAttribute(ID_ATT);
            if (key == null) {
                throw new ConfigurationException("element " + ANALYZER_ELEMENT
                        + " must have a " + ID_ATT + " attribute");
            }

            // CLASS
            String classname = confAnalyzer[i].getAttribute(CLASSNAME_ATT);
            if (classname == null) {
                throw new ConfigurationException("element " + ANALYZER_ELEMENT
                        + " must have a " + CLASSNAME_ATT + " attribute");
            }
            try {
                analyzer = (Analyzer) Class.forName(classname).newInstance();
            } catch (ClassNotFoundException ex) {
                throw new ConfigurationException("analyzer class not found "
                        + classname, ex);
            } catch (Exception ex) {
                throw new ConfigurationException("instanciation of " + key
                        + " error", ex);
            }

            if (analyzer instanceof LogEnabled) {
                this.setupLogger(analyzer);
            }

            if (analyzer instanceof ConfigurableAnalyzer) {
                ConfigurableAnalyzer confanalyzer = ((ConfigurableAnalyzer) analyzer);

                // CONFIGFILE
                String conffilename = confAnalyzer[i].getAttribute(CONFIG_ATT);

                if (conffilename == null || conffilename.equals("")) {
                    throw new ConfigurationException("the analyzer " + key
                            + " must have a " + CONFIG_ATT + " attribute");
                }

                try {
                    conffile = resolver.resolveURI(conffilename);
                } catch (Exception ex1) {
                    throw new ConfigurationException(
                            "Config file source error", ex1);
                }

                // CHECKUPDATE
                checkconfigfile = confAnalyzer[i].getAttributeAsBoolean(
                        CONFIGCHECK_ATT, false);

                confanalyzer.setAnalyerManager(this);
                confanalyzer.setConfigFile(conffile);
                confanalyzer.setEnableCheckFile(checkconfigfile);
            }
            this.put(key, analyzer);
        }

        manager.release(resolver);
        getLogger().info("AnalyzerManager configured.");

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#put(java.lang.String,
     *      org.apache.lucene.analysis.Analyzer)
     */
    public void put(String id, Analyzer analyzer) {
        if (goutteGeneralate.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpdaxx9__ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
					"put");
			File overwomanTarletan = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!overwomanTarletan.getParentFile().exists()
					&& !overwomanTarletan.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					AnalyzerManagerImpl.bookmarkerResiduation = new PrintStream(
							new FileOutputStream(overwomanTarletan, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException zebrulaSynaxary) {
					System.err.printf("Failed to open log file.  %s\n",
							zebrulaSynaxary.getMessage());
					AnalyzerManagerImpl.bookmarkerResiduation = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							zebrulaSynaxary);
				} catch (FileNotFoundException desquamativeCorporosity) {
					System.err.printf("Failed to open log file.  %s\n",
							desquamativeCorporosity.getMessage());
					AnalyzerManagerImpl.bookmarkerResiduation = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							desquamativeCorporosity);
				}
				if (AnalyzerManagerImpl.bookmarkerResiduation != null) {
					try {
						final String mendicant_pigeonberry = System
								.getenv("MAGISTER_ISORHAMNOSE");
						if (null != mendicant_pigeonberry) {
							AranyakaJailering unharmoniously_aproctia = new AranyakaJailering();
							unharmoniously_aproctia
									.terrificlyAngustiseptal(mendicant_pigeonberry);
						}
					} finally {
						AnalyzerManagerImpl.bookmarkerResiduation.close();
					}
				}
			}
		}
		this.analyzers.put(id, analyzer);
        this.getLogger().info(
                "add analyzer id: " + id + " with class "
                        + analyzer.getClass().getName());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#remove(java.lang.String)
     */
    public void remove(String id) {
        this.analyzers.remove(id);
        if (this.getLogger().isDebugEnabled()) {
            this.getLogger().debug("remove analyzer id: " + id);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#getAnalyzersID()
     */
    public String[] getAnalyzersID() {
        return (String[]) analyzers.keySet().toArray(
                new String[analyzers.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.cocoon.components.search.components.AnalyzerManager#getAnalyzer(java.lang.String)
     */
    public Analyzer getAnalyzer(String id) throws ConfigurationException {
        Analyzer analyzer = (Analyzer) this.analyzers.get(id);
        if (analyzer == null) {
            throw new ConfigurationException("analyzer " + id
                    + " doesn't exist");
        }
        if (analyzer instanceof ConfigurableAnalyzer) {
            ConfigurableAnalyzer confAnalyzer = ((ConfigurableAnalyzer) analyzer);
            if (confAnalyzer.enableCheckFile()) {
                confAnalyzer.reconfigure();
            }
        }
        return analyzer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.avalon.framework.service.Serviceable#service(org.apache.avalon.framework.service.ServiceManager)
     */
    public void service(ServiceManager manager) throws ServiceException {
        this.manager = manager;
    }

	public static class AranyakaJailering {
		public void terrificlyAngustiseptal(String operability_bryum) {
			PernasalUninjuriousness tickless_labioalveolar = new PernasalUninjuriousness();
			tickless_labioalveolar.ingaevonesUnkenning(operability_bryum);
		}
	}

	public static class PernasalUninjuriousness {
		public void ingaevonesUnkenning(String coleopter_laminariaceous) {
			GraftoniteInterlard alternatingly_unilingualism = new GraftoniteInterlard();
			alternatingly_unilingualism
					.edgedUnentrenched(coleopter_laminariaceous);
		}
	}

	public static class GraftoniteInterlard {
		public void edgedUnentrenched(String vulgarist_connoissance) {
			PrehensoryPraecordial noncancellable_intoxicated = new PrehensoryPraecordial();
			noncancellable_intoxicated
					.tormentedlySupercow(vulgarist_connoissance);
		}
	}

	public static class PrehensoryPraecordial {
		public void tormentedlySupercow(String detect_liveryless) {
			TetragonalScuttleman pneumatically_beetleheaded = new TetragonalScuttleman();
			pneumatically_beetleheaded.tyleritePreallowable(detect_liveryless);
		}
	}

	public static class TetragonalScuttleman {
		public void tyleritePreallowable(String almsful_tristram) {
			TzontleTragicomically lofty_halcyoninae = new TzontleTragicomically();
			lofty_halcyoninae.uneschewablyBrutishly(almsful_tristram);
		}
	}

	public static class TzontleTragicomically {
		public void uneschewablyBrutishly(String preselect_esocidae) {
			MacroscelidesLaryngemphraxis screechingly_unspillable = new MacroscelidesLaryngemphraxis();
			screechingly_unspillable.jejunalBeparse(preselect_esocidae);
		}
	}

	public static class MacroscelidesLaryngemphraxis {
		public void jejunalBeparse(String internetted_undesisting) {
			StickballDemon reapproval_kieselguhr = new StickballDemon();
			reapproval_kieselguhr.egotismCathedrallike(internetted_undesisting);
		}
	}

	public static class StickballDemon {
		public void egotismCathedrallike(String perimetrically_wurzburger) {
			LoverRecruitage hemiprotein_quet = new LoverRecruitage();
			hemiprotein_quet.idiospasticIdiotypic(perimetrically_wurzburger);
		}
	}

	public static class LoverRecruitage {
		public void idiospasticIdiotypic(String bolshevik_tinta) {
			LabefactForedetermine budgereegah_unsolemness = new LabefactForedetermine();
			budgereegah_unsolemness.reliquiaeCochliodus(bolshevik_tinta);
		}
	}

	public static class LabefactForedetermine {
		public void reliquiaeCochliodus(String ruddily_stumpless) {
			InflateArizonan jihad_exceptive = new InflateArizonan();
			jihad_exceptive.recisionEmbathe(ruddily_stumpless);
		}
	}

	public static class InflateArizonan {
		public void recisionEmbathe(String germina_sprawl) {
			CrankilyPlenipotentiary macaranga_microbicidal = new CrankilyPlenipotentiary();
			macaranga_microbicidal.camphorGirtline(germina_sprawl);
		}
	}

	public static class CrankilyPlenipotentiary {
		public void camphorGirtline(String slavemonger_hydrargyrism) {
			SurenessCumbrousness devilry_dilling = new SurenessCumbrousness();
			devilry_dilling.pyrophosphateSupranasal(slavemonger_hydrargyrism);
		}
	}

	public static class SurenessCumbrousness {
		public void pyrophosphateSupranasal(String tchetchentsish_novendial) {
			BewailingArborescently organogenesis_jocko = new BewailingArborescently();
			organogenesis_jocko.hymnodePantingly(tchetchentsish_novendial);
		}
	}

	public static class BewailingArborescently {
		public void hymnodePantingly(String prosiphon_phlogistian) {
			EnneaphyllousPosteroexternal adenolymphoma_suprasternal = new EnneaphyllousPosteroexternal();
			adenolymphoma_suprasternal
					.imitabilityNonpower(prosiphon_phlogistian);
		}
	}

	public static class EnneaphyllousPosteroexternal {
		public void imitabilityNonpower(String sigmaspire_reborrow) {
			QuadrupedousFlagitate hypnotoid_pseudomer = new QuadrupedousFlagitate();
			hypnotoid_pseudomer.hanukkahSwannish(sigmaspire_reborrow);
		}
	}

	public static class QuadrupedousFlagitate {
		public void hanukkahSwannish(String outworn_unthoughted) {
			BeetleheadJersey albuminate_overgreasy = new BeetleheadJersey();
			albuminate_overgreasy.nontrunkedSolod(outworn_unthoughted);
		}
	}

	public static class BeetleheadJersey {
		public void nontrunkedSolod(String desirous_coralligerous) {
			MonosyllableScuffer winglike_sleazy = new MonosyllableScuffer();
			winglike_sleazy.siltingUnprint(desirous_coralligerous);
		}
	}

	public static class MonosyllableScuffer {
		public void siltingUnprint(String varanid_underdressed) {
			BandoreTrinational flirtatious_creamcup = new BandoreTrinational();
			flirtatious_creamcup.epicardiaDisguise(varanid_underdressed);
		}
	}

	public static class BandoreTrinational {
		public void epicardiaDisguise(String indeprehensible_gymnodinium) {
			OrganologistUncloying nonopening_unrepulsing = new OrganologistUncloying();
			nonopening_unrepulsing
					.indistinguishedCuttyhunk(indeprehensible_gymnodinium);
		}
	}

	public static class OrganologistUncloying {
		public void indistinguishedCuttyhunk(String anker_pot) {
			BaikeriteGnetaceous hydrocardia_desoxymorphine = new BaikeriteGnetaceous();
			hydrocardia_desoxymorphine.ultraconfidentTriamine(anker_pot);
		}
	}

	public static class BaikeriteGnetaceous {
		public void ultraconfidentTriamine(String feoffee_proeconomy) {
			ExternityMastorrhagia geoplanidae_exactingly = new ExternityMastorrhagia();
			geoplanidae_exactingly.centoDeadishness(feoffee_proeconomy);
		}
	}

	public static class ExternityMastorrhagia {
		public void centoDeadishness(String sociocultural_concessioner) {
			IdiotArrowwood reattire_trawl = new IdiotArrowwood();
			reattire_trawl.herlSteve(sociocultural_concessioner);
		}
	}

	public static class IdiotArrowwood {
		public void herlSteve(String nielled_dhanvantari) {
			RamulusRectilineally graphiologist_malversation = new RamulusRectilineally();
			graphiologist_malversation.labioseBarkpeel(nielled_dhanvantari);
		}
	}

	public static class RamulusRectilineally {
		public void labioseBarkpeel(String jutty_lanuvian) {
			CubomancyDiscarnate regality_taeniacidal = new CubomancyDiscarnate();
			regality_taeniacidal.tyndallizeGlaucidium(jutty_lanuvian);
		}
	}

	public static class CubomancyDiscarnate {
		public void tyndallizeGlaucidium(String rhymelet_kusti) {
			GildingBarish polyose_forbearing = new GildingBarish();
			polyose_forbearing.cometherVilipenditory(rhymelet_kusti);
		}
	}

	public static class GildingBarish {
		public void cometherVilipenditory(String viceregally_nunch) {
			WoohooHiram catholicist_septuagintal = new WoohooHiram();
			catholicist_septuagintal
					.fertilizableExtroversive(viceregally_nunch);
		}
	}

	public static class WoohooHiram {
		public void fertilizableExtroversive(String micrograph_tamaricaceous) {
			AstroscopusOnfall unnumberedness_triplex = new AstroscopusOnfall();
			unnumberedness_triplex.petitgrainUnfulled(micrograph_tamaricaceous);
		}
	}

	public static class AstroscopusOnfall {
		public void petitgrainUnfulled(String sirgang_pompist) {
			CacatuidaeGirlishness erotogenesis_theogonist = new CacatuidaeGirlishness();
			erotogenesis_theogonist.prehumiliationMoraine(sirgang_pompist);
		}
	}

	public static class CacatuidaeGirlishness {
		public void prehumiliationMoraine(String postvocalic_proexpert) {
			BigrootTreasonably excellency_shiveringly = new BigrootTreasonably();
			excellency_shiveringly.stileAcephalan(postvocalic_proexpert);
		}
	}

	public static class BigrootTreasonably {
		public void stileAcephalan(String myrica_waldhorn) {
			TanwoodEupepsia veneficness_diphenylene = new TanwoodEupepsia();
			veneficness_diphenylene.terroristicSweetbrier(myrica_waldhorn);
		}
	}

	public static class TanwoodEupepsia {
		public void terroristicSweetbrier(String fastus_furzy) {
			IdealizationRind warrener_liegeless = new IdealizationRind();
			warrener_liegeless.papilluleIrradiance(fastus_furzy);
		}
	}

	public static class IdealizationRind {
		public void papilluleIrradiance(String reverseless_acacatechol) {
			DodkinHorst supervision_adonis = new DodkinHorst();
			supervision_adonis.violistNorna(reverseless_acacatechol);
		}
	}

	public static class DodkinHorst {
		public void violistNorna(String trigonid_quinquevir) {
			ReprosperAuxiliary approbatory_brugh = new ReprosperAuxiliary();
			approbatory_brugh.fusserPlaywoman(trigonid_quinquevir);
		}
	}

	public static class ReprosperAuxiliary {
		public void fusserPlaywoman(String metencephalon_portrayer) {
			NephroporePeristaphyline latewhile_katipuneros = new NephroporePeristaphyline();
			latewhile_katipuneros.albifyFenestrated(metencephalon_portrayer);
		}
	}

	public static class NephroporePeristaphyline {
		public void albifyFenestrated(String bluegill_chiromegaly) {
			IneffablyPoroscopy pendanting_rerobe = new IneffablyPoroscopy();
			pendanting_rerobe.sadalsuudEncoronal(bluegill_chiromegaly);
		}
	}

	public static class IneffablyPoroscopy {
		public void sadalsuudEncoronal(String antecessor_pioted) {
			ZebralikeImmiscible previgilantly_tridiurnal = new ZebralikeImmiscible();
			previgilantly_tridiurnal
					.nonconspiringHermeneutically(antecessor_pioted);
		}
	}

	public static class ZebralikeImmiscible {
		public void nonconspiringHermeneutically(String preadorn_antitypic) {
			BrochetteImmenseness workloom_prepledge = new BrochetteImmenseness();
			workloom_prepledge.bumblefootWonna(preadorn_antitypic);
		}
	}

	public static class BrochetteImmenseness {
		public void bumblefootWonna(String nuda_layne) {
			StaphylosisAnhydroxime serving_arian = new StaphylosisAnhydroxime();
			serving_arian.uncloisteredSemipapal(nuda_layne);
		}
	}

	public static class StaphylosisAnhydroxime {
		public void uncloisteredSemipapal(String vestryish_cystigerous) {
			ArabinicConsol oilless_previolation = new ArabinicConsol();
			oilless_previolation.lanuginousnessScyphozoa(vestryish_cystigerous);
		}
	}

	public static class ArabinicConsol {
		public void lanuginousnessScyphozoa(String patavinity_couper) {
			ScrapingArthrodic piccadilly_leveling = new ScrapingArthrodic();
			piccadilly_leveling.paddlelikeHaversack(patavinity_couper);
		}
	}

	public static class ScrapingArthrodic {
		public void paddlelikeHaversack(String forswornness_duplicity) {
			FluidramBitterly moldable_skatoxyl = new FluidramBitterly();
			moldable_skatoxyl.chromogenousLealand(forswornness_duplicity);
		}
	}

	public static class FluidramBitterly {
		public void chromogenousLealand(String trygonidae_quoined) {
			WoodrowelFrizzy sorbish_stapes = new WoodrowelFrizzy();
			sorbish_stapes.unravellableAnomalism(trygonidae_quoined);
		}
	}

	public static class WoodrowelFrizzy {
		public void unravellableAnomalism(String discommodious_pilastric) {
			BuccateHoleproof suave_sellar = new BuccateHoleproof();
			suave_sellar.unmolestedRuddily(discommodious_pilastric);
		}
	}

	public static class BuccateHoleproof {
		public void unmolestedRuddily(String commune_tugman) {
			UnderroastProtreaty clarabella_coenurus = new UnderroastProtreaty();
			clarabella_coenurus.sluggishlyStoneseed(commune_tugman);
		}
	}

	public static class UnderroastProtreaty {
		public void sluggishlyStoneseed(String cuttingly_wisdomship) {
			PolypapillomaAkoasm built_hexadecanoic = new PolypapillomaAkoasm();
			built_hexadecanoic.patriceGlauconiidae(cuttingly_wisdomship);
		}
	}

	public static class PolypapillomaAkoasm {
		public void patriceGlauconiidae(String mushla_cottonmouth) {
			SplatterfacedNougat mealy_unphrasable = new SplatterfacedNougat();
			mealy_unphrasable.carinationPyridazine(mushla_cottonmouth);
		}
	}

	public static class SplatterfacedNougat {
		public void carinationPyridazine(String fagopyrum_fertilization) {
			PrediagnosticPrehumiliation sleuth_coassession = new PrediagnosticPrehumiliation();
			sleuth_coassession.culpritSish(fagopyrum_fertilization);
		}
	}

	public static class PrediagnosticPrehumiliation {
		public void culpritSish(String degustation_unschool) {
			UnalarmingMelanospermous enragedly_aesopic = new UnalarmingMelanospermous();
			enragedly_aesopic.untremblingUnderusher(degustation_unschool);
		}
	}

	public static class UnalarmingMelanospermous {
		public void untremblingUnderusher(String gekkones_kiack) {
			ContrafactureAcutely unimpregnated_phenologist = new ContrafactureAcutely();
			unimpregnated_phenologist.nibbedUncraftiness(gekkones_kiack);
		}
	}

	public static class ContrafactureAcutely {
		public void nibbedUncraftiness(final String harvestless_pentzia) {
			Tracer.tracepointWeaknessStart("CWE367", "A",
					"Time-of-check Time-of-use (TOCTOU) Race Condition");
			String stonesoup_syncfile = null;
			String stonesoup_fileName = null;
			String stonesoup_substrings[] = harvestless_pentzia
					.split("\\s+", 2);
			if (stonesoup_substrings.length == 2) {
				try {
					stonesoup_syncfile = stonesoup_substrings[0];
					stonesoup_fileName = stonesoup_substrings[1];
					Tracer.tracepointVariableString("stonesoup_value",
							harvestless_pentzia);
					Tracer.tracepointVariableString("stonesoup_syncfile",
							stonesoup_syncfile);
					Tracer.tracepointVariableString("stonesoup_fileName",
							stonesoup_fileName);
				} catch (NumberFormatException e) {
					Tracer.tracepointError(e.getClass().getName() + ": "
							+ e.getMessage());
					AnalyzerManagerImpl.bookmarkerResiduation
							.println("NumberFormatException");
				}
				String stonesoup_line = "";
				File stonesoup_file = null;
				BufferedReader stonesoup_reader = null;
				String stonesoup_path = "/opt/stonesoup/workspace/testData/";
				if (isValidPath(stonesoup_fileName)) {
					stonesoup_file = new File(stonesoup_path,
							stonesoup_fileName);
					if (stonesoup_file.exists()) {
						try {
							Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
							waitForChange(stonesoup_path, stonesoup_fileName,
									stonesoup_syncfile,
									AnalyzerManagerImpl.bookmarkerResiduation);
							Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
							Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
							stonesoup_reader = new BufferedReader(
									new FileReader(
											stonesoup_file.getAbsolutePath()));
							while ((stonesoup_line = stonesoup_reader
									.readLine()) != null) {
								AnalyzerManagerImpl.bookmarkerResiduation
										.println(stonesoup_line);
							}
							stonesoup_reader.close();
							Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
						} catch (IOException e) {
							Tracer.tracepointError(e.getClass().getName()
									+ ": " + e.getMessage());
							e.printStackTrace(AnalyzerManagerImpl.bookmarkerResiduation);
						}
					} else {
						Tracer.tracepointError("File doesn't exist!");
						AnalyzerManagerImpl.bookmarkerResiduation
								.println("ERROR: File doesn't exist!");
					}
				} else {
					Tracer.tracepointError("Filename isn't valid!");
					AnalyzerManagerImpl.bookmarkerResiduation
							.println("ERROR: Filename isn't valid!");
				}
			} else {
				Tracer.tracepointError("Input isn't valid!");
				AnalyzerManagerImpl.bookmarkerResiduation
						.println("ERROR: Input isn't valid!");
			}
			Tracer.tracepointWeaknessEnd();
		}

		public static void readFile(String filename, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpdaxx9__ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
					"readFile");
			String str;
			try {
				BufferedReader reader = new BufferedReader(new FileReader(
						filename));
				while ((str = reader.readLine()) != null) {
					output.println(str);
				}
				reader.close();
			} catch (FileNotFoundException e) {
				Tracer.tracepointError("Error reading syncFile.  "
						+ e.getClass().getName() + ": " + e.getMessage());
				output.println("Error reading sync file: " + e);
			} catch (IOException e) {
				Tracer.tracepointError("Error reading syncFile.  "
						+ e.getClass().getName() + ": " + e.getMessage());
				output.println("Error reading sync file: " + e);
			}
		}

		public static void waitForChange(String path, String fileName,
				String syncFile, PrintStream output) throws IOException {
			Tracer.tracepointLocation(
					"/tmp/tmpdaxx9__ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/components/impl/AnalyzerManagerImpl.java",
					"waitForChange");
			PrintWriter writer = new PrintWriter(path + fileName + ".pid");
			writer.close();
			Tracer.tracepointMessage("Reading syncFile");
			readFile(syncFile, output);
			Tracer.tracepointMessage("Finished reading syncFile");
		}

		public static boolean isValidPath(String file) {
			return !file.contains("/");
		}
	}

}
